@@ -20,25 +20,78 @@ Driver base-classes:
     types that support that contract
 """
 
+import dataclasses
 import itertools
 import sys
+import time
 import typing as ty
 
 import os_resource_classes as orc
 import os_traits
+
 from oslo_log import log as logging
 from oslo_utils import importutils
 
 import nova.conf
+import nova.virt.node
+
 from nova import context as nova_context
 from nova.i18n import _
+from nova.network import model as network_model
 from nova import objects
+from nova import version
 from nova.virt import event as virtevent
 
 CONF = nova.conf.CONF
 LOG = logging.getLogger(__name__)
 
 
+@dataclasses.dataclass
+class FlavorMeta:
+    name: str
+    memory_mb: int
+    vcpus: int
+    root_gb: int
+    ephemeral_gb: int
+    extra_specs: dict
+    swap: int
+
+
+@dataclasses.dataclass
+class ImageMeta:
+    id: str
+    name: str
+    properties: dict
+
+
+@dataclasses.dataclass
+class NovaInstanceMeta:
+    name: str
+    uuid: str
+
+
+@dataclasses.dataclass
+class OwnerMeta:
+    userid: str
+    username: str
+    projectid: str
+    projectname: str
+
+
+@dataclasses.dataclass
+class InstanceDriverMetadata:
+    root_type: str
+    root_id: str
+    instance_meta: NovaInstanceMeta
+    owner: OwnerMeta
+    image: ImageMeta
+    flavor: FlavorMeta
+    network_info: network_model.NetworkInfo
+    nova_package: str = dataclasses.field(
+        default_factory=version.version_string_with_package)
+    creation_time: float = dataclasses.field(default_factory=time.time)
+
+
 def get_block_device_info(instance, block_device_mapping):
     """Converts block device mappings for an instance to driver format.
 
@@ -108,10 +161,16 @@ def block_device_info_get_encrypted_disks(
     block_device_info: ty.Mapping[str, ty.Any],
 ) -> ty.List['nova.virt.block_device.DriverBlockDevice']:
     block_device_info = block_device_info or {}
+
+    # swap is a single device, not a list
+    swap = block_device_info.get('swap')
+    swap = [swap] if swap else []
+
     return [
         driver_bdm for driver_bdm in itertools.chain(
             block_device_info.get('image', []),
             block_device_info.get('ephemerals', []),
+            swap,
         )
         if driver_bdm.get('encrypted')
     ]
@@ -156,6 +215,12 @@ CAPABILITY_TRAITS_MAP = {
         os_traits.COMPUTE_EPHEMERAL_ENCRYPTION_LUKS,
     "supports_ephemeral_encryption_plain":
         os_traits.COMPUTE_EPHEMERAL_ENCRYPTION_PLAIN,
+    "supports_address_space_passthrough":
+        os_traits.COMPUTE_ADDRESS_SPACE_PASSTHROUGH,
+    "supports_address_space_emulated":
+        os_traits.COMPUTE_ADDRESS_SPACE_EMULATED,
+    "supports_stateless_firmware":
+        os_traits.COMPUTE_SECURITY_STATELESS_FIRMWARE,
 }
 
 
@@ -225,6 +290,8 @@ class ComputeDriver(object):
         "supports_secure_boot": False,
         "supports_socket_pci_numa_affinity": False,
         "supports_remote_managed_ports": False,
+        "supports_address_space_passthrough": False,
+        "supports_address_space_emulated": False,
 
         # Ephemeral encryption support flags
         "supports_ephemeral_encryption": False,
@@ -281,6 +348,55 @@ class ComputeDriver(object):
         # TODO(Vek): Need to pass context in for access to auth_token
         raise NotImplementedError()
 
+    @classmethod
+    def get_instance_driver_metadata(
+        cls, instance: 'nova.objects.instance.Instance',
+        network_info: network_model.NetworkInfo
+    ) -> InstanceDriverMetadata:
+        """Get driver metadata from instance and network info
+
+        :param instance: nova.objects.instance.Instance
+        :param network_info: instance network information
+        :returns: InstanceDriverMetadata
+        """
+
+        instance_name = instance.display_name or instance.uuid
+        system_meta = instance.system_metadata
+        instance_meta = NovaInstanceMeta(
+            str(instance_name), str(instance.uuid))
+        owner = OwnerMeta(
+            userid=instance.user_id,
+            username=system_meta.get('owner_user_name', 'N/A'),
+            projectid=instance.project_id,
+            projectname=system_meta.get('owner_project_name', 'N/A')
+        )
+        flavor = FlavorMeta(
+            name=instance.flavor.name,
+            memory_mb=instance.flavor.memory_mb,
+            vcpus=instance.flavor.vcpus,
+            ephemeral_gb=instance.flavor.ephemeral_gb,
+            root_gb=instance.flavor.root_gb,
+            swap=instance.flavor.swap,
+            extra_specs=instance.flavor.extra_specs,
+        )
+        image = ImageMeta(
+            id=instance.image_ref,
+            name=system_meta.get('image_name'),
+            properties=instance.image_meta.properties
+        )
+        meta = InstanceDriverMetadata(
+            instance_meta=instance_meta,
+            owner=owner,
+            flavor=flavor,
+            image=image,
+            root_type = 'image' if instance.image_ref else 'volume',
+            root_id = instance.image_ref,
+            creation_time = time.time(),
+            network_info=network_info
+        )
+        LOG.debug('InstanceDriverMetadata: %s', meta)
+        return meta
+
     def get_num_instances(self):
         """Return the total number of virtual machines.
 
@@ -540,16 +656,6 @@ class ComputeDriver(object):
         """
         raise NotImplementedError()
 
-    def get_rdp_console(self, context, instance):
-        """Get connection info for a rdp console.
-
-        :param context: security context
-        :param instance: nova.objects.instance.Instance
-
-        :returns: an instance of console.type.ConsoleRDP
-        """
-        raise NotImplementedError()
-
     def get_serial_console(self, context, instance):
         """Get connection info for a serial console.
 
@@ -611,7 +717,7 @@ class ComputeDriver(object):
                       disk_bus=None, device_type=None, encryption=None):
         """Attach the disk to the instance at mountpoint using info.
 
-        :raises TooManyDiskDevices: if the maxmimum allowed devices to attach
+        :raises TooManyDiskDevices: if the maximum allowed devices to attach
                                     to a single instance is exceeded.
         """
         raise NotImplementedError()
@@ -1046,7 +1152,7 @@ class ComputeDriver(object):
         node, as well as the inventory, aggregates, and traits associated with
         those resource providers.
 
-        Implementors of this interface are expected to set ``allocation_ratio``
+        Implementers of this interface are expected to set ``allocation_ratio``
         and ``reserved`` values for inventory records, which may be based on
         configuration options, e.g. ``[DEFAULT]/cpu_allocation_ratio``,
         depending on the driver and resource class. If not provided, allocation
@@ -1153,6 +1259,26 @@ class ComputeDriver(object):
         """
         raise NotImplementedError()
 
+    def is_node_deleted(self, nodename):
+        """Check this compute node has been deleted.
+
+        This method is called when the compute manager notices that a
+        node that was previously reported as available is no longer
+        available.
+        In this case, we need to know if the node has actually been
+        deleted, or if it is simply no longer managed by this
+        nova-compute service.
+        If True is returned, the database and placement records will
+        be removed.
+
+        :param nodename:
+            node which the caller wants to check if its deleted
+        :returns: True if the node is safe to delete
+        """
+        # For most driver, compute nodes should never get deleted
+        # during a periodic task, they are only deleted via the API
+        return False
+
     def pre_live_migration(self, context, instance, block_device_info,
                            network_info, disk_info, migrate_data):
         """Prepare an instance for live migration
@@ -1164,7 +1290,7 @@ class ComputeDriver(object):
         :param disk_info: instance disk information
         :param migrate_data: a LiveMigrateData object
         :returns: migrate_data modified by the driver
-        :raises TooManyDiskDevices: if the maxmimum allowed devices to attach
+        :raises TooManyDiskDevices: if the maximum allowed devices to attach
                                     to a single instance is exceeded.
         """
         raise NotImplementedError()
@@ -1315,6 +1441,22 @@ class ComputeDriver(object):
         """
         raise NotImplementedError()
 
+    def check_source_migrate_data_at_dest(self, ctxt, instance, migrate_data,
+                                         migration, limits, allocs):
+        """Runs the last checks on the destination after the source returned
+        the migrate_data.
+
+        :param ctxt: security context
+        :param instance: nova.db.main.models.Instance
+        :param migrate_data: result of check_can_live_migrate_source
+        :param migration: The Migration object for this live migration
+        :param limits: The SchedulerLimits object for this live migration
+        :param allocs: Allocations for this instance
+        :returns: a LibvirtLiveMigrateData object
+        :raises: MigrationPreCheckError
+        """
+        return migrate_data
+
     def post_claim_migrate_data(self, context, instance, migrate_data, claim):
         """Returns migrate_data augmented with any information obtained from
         the claim. Intended to run on the destination of a live-migration
@@ -1595,6 +1737,11 @@ class ComputeDriver(object):
         """
         raise NotImplementedError()
 
+    def get_nodenames_by_uuid(self, refresh=False):
+        """Returns a dict of {uuid: nodename} for all managed nodes."""
+        nodename = self.get_available_nodes()[0]
+        return {nova.virt.node.get_local_node_uuid(): nodename}
+
     def node_is_available(self, nodename):
         """Return whether this compute service manages a particular node."""
         if nodename in self.get_available_nodes():
@@ -1713,7 +1860,7 @@ class ComputeDriver(object):
             The metadata of the image of the instance.
         :param nova.objects.BlockDeviceMapping root_bdm:
             The description of the root device.
-        :raises TooManyDiskDevices: if the maxmimum allowed devices to attach
+        :raises TooManyDiskDevices: if the maximum allowed devices to attach
                                     to a single instance is exceeded.
         """
         raise NotImplementedError()
@@ -1722,7 +1869,7 @@ class ComputeDriver(object):
                                           *block_device_lists):
         """Default the missing device names in the block device mapping.
 
-        :raises TooManyDiskDevices: if the maxmimum allowed devices to attach
+        :raises TooManyDiskDevices: if the maximum allowed devices to attach
                                     to a single instance is exceeded.
         """
         raise NotImplementedError()
@@ -1741,7 +1888,7 @@ class ComputeDriver(object):
                                  implementation if not set.
 
         :returns: The chosen device name.
-        :raises TooManyDiskDevices: if the maxmimum allowed devices to attach
+        :raises TooManyDiskDevices: if the maximum allowed devices to attach
                                     to a single instance is exceeded.
         """
         raise NotImplementedError()
